
/****************************************************************************
*** NOME       : Vetores.prg
*** UTILIZAO : Salvar, ler e restaurar vetores no disco.
*** INCIO     : 16/12/2001
Autor: Steve Straley
Extraido do livro Programao Avanada em Clipper 5.2 - Stephen J. Straley
Berkeley Brasil Editora - ISBN 85-7251-241-1 - Edio em portugus esgotada.
Disponvel em ingls na livraria Cultura em SP ou www.livrariacultura.com.br

*** Observao : Como vetores no Clipper podem ser multidimensionais, o mtodo
para salva-los no disco deve levar isso isso em conta.

O Clipper depende de dois fatos para implementar vetores multidimensionais:
vetores podem conter qualquer tipo de dado vlido como um elemento em um vetor,
e um vetor , por s s, um tipo de dado vlido. Isso significa que um vetor
multidimensional nada mais  que um vetor em um vetor. A funo SaveArray()
usa este fato para chamar a s propria recursivamente enquanto est rodando.

Dentro do bloco de cdigo processado por essa funo a funo WriteArray() 
chamada recursivamente. Com o advento de variveis locais, este  um mtodo
prtico para gravar um vetor no disco, especialmente quando os vetores possam
ser desiguais, podem conter outros vetores e podem ser de tipos de dados
mistos.

Para os crticos alerto que ela encontrar problemas depois da 50 chamada
recursiva  funo WhiteArray(), contudo os vetores que manipulamos tem, no
mximo, 4 ou 5 nveis de profundidade, portanto, bem dentro das nossas
necessidades.

****************************************************************************/

#include "ptverbs.ch"
#include "ptvalue.ch"
#include "ptfuncs.ch"
#include "ptcolor.ch"
#include "ptinkey.ch"

Set Epoch to 1950
Set Century On
Set Date British

// Local aArray:={{1,2,3},4,5,{6,7},{8,9}}
// aArray local:={1,2,3,4,5,6,7,8,9}
// aArray local:={1}

******************
Function SaveArray
******************
Parameters aArray,cFileName
Local lRetValue:=.f.
Local nFhandle

If aArray Is pArray
   Id cFileName Is pCharacter
      nFhandle:=Fcreate(cFileName)
      If nFhandle > 4
         If WriteArray(nFhandle,aArray)
            If Empty(Ferror())
               lRetValue:=pTrue
            Endif
         Endif
      Endif
      Fclose(nFhandle)
   Endif
Endif
Return(lRetValue)

*** Fim - Function SaveArray

**************************
Static Function WriteArray
**************************
Parameters nHandle,xItem
Local lRetValue:=.f.
Local cDataType:=ValType(xItem)
Local nLength
Local cString
Fwrite(nHandle,cDataType,1)
If Empty(Ferror())
   Do Case
   Case cDatatype == "A"
      nLength:=Len(xItem)
      Fwrite(nHandle,L2bin(nLenght),4)    // "L" de laranja
      Aeval(xItem,{ | xNewItem | WriteArray(nHandle,xNewItem)})
   Case cDatatype == "C"
      Fwrite(nHandle, L2bin( Len(xItem)),4)
      Fwrite(nHandle, xItem, Len(xItem))
   Case cDatatype == "D"
      Fwrite(nHandle, L2bin( 8),4)
      Fwrite(nHandle, Dtoc(xItem),8)
   Case cDatatype == "L"
      Fwrite(nHandle, L2bin( 1),4)
      Fwrite(nHandle, If(xItem,.t. , .f.))
   Case cDatatype == "N"
      nLength:=Len(cString:=Str(xItem))
      Fwrite(nHandle, L2bin(nLenght),4)
      Fwrite(nHandle, cString, nLenght)
   OtherWise // Bloco de cdigo, Nil ou Objeto
      Fwrite(nHandle, L2bin(3),4)
      Fwrite(nhandle, "Nil")
   EndCase
   lRetValue:=Empty(Ferror())
Endif
Return(lRetValue)

*** Fim - Function WriteArray

****************************
Static Function RestoreArray
****************************
Parameters cFileName
Local nFhandle
Local aRetValue:={}

If cFileName Is pCharacter
  If File(cFileName)
     nFhandle:=Fopen(cFileName)
     If nFhandle > 4
        aRetValue:=ReadArray(nFhandle)
     Endif
  Endif
Endif
Return(aRetValue)

*** Fim - Function RestoreArray

*************************
Static Function ReadArray
*************************
Parameters nHandle
Local cSize:= Space(4)
Local cDataType:= Space(1)
Local nLength
Local xItem
Local nCounter
Local cBuffer

Fread(nHandle, @cDataType,1)
Fread(nHandle, @cSize,4)
nLenght:=Bin2L(cSize)
If cDataType == "A"
   xItem:={}
   For nCounter:=1 to nLenght
      Aadd(xItem,ReadArray(nHandle))
   Next
Else
   cBuffer:=Space(nLenght)
   Fread(nHandle,@cBuffer,nLenght)
   Do Case
   Case cDataType == pCharacter
      xItem:= cBuffer
   Case cDataType == pDate
      xItem:= Ctod(cBuffer)
   Case cDataType == pLogical
      xItem:= (cBuffer == "T")
   Case cDataType == pBlock
      xItem:= Nil
   Case cDataType == pNumeric
      xItem:= Val(cBuffer)
   EndCase
Endif
Return(xItem)

*** Fim - Function ReadArray

*
* Fim do programa Vetores.prg
*

